/**
 * Open PowerShell Plugin for WeBuilder
 * Version: 1.26 beta
 * Last Changes: 12.07.2016
 * by Frank Butenhoff <frank@butenhoff.net>
 * license: http://www.freebsd.org/copyright/license.html  BSD License
 */

/**
 * Global variables
 */
var currentFileName,
    currentDirName,
    fileEnc,
    onlyFileName,
    fileName,
    fileType,
    bitmap,
    actionPic;

bitmap = new TBitmap;

/**
 * Init Plugin
 */
function initPlugin() {
    fileEnc = Document.Encoding;
    fileName = Document.FileName;
    onlyFileName = ExtractFileName(fileName);
    fileType  = ExtractFileExt(fileName);
    currentDirName = ExtractFilePath(fileName);
}

/**
 * test on file exists
 */
function getFileExists( file ) {
    if (file != "") {
        return FileExists(file);
    }
    return false;
}

/**
 * Test on right file extension
 */
function getFileExt(fileExt) {
    var okay = false,
        arr = "";
    arr = split(fileExt, ",");
    for (var i = 0; i < length(arr); i++) {
        if (arr[i] == fileType) {
            okay = true;
        }
    }
    if (okay) {
        return true;
    }
    return false;
}

/**
 * Open only Powershell
 */
function powerShell() {
    ExecuteFile(Script.Path + "ps.cmd", "\""+currentDirName+"\"");
}

/**
 * Open Powershell with required file on workdir
 */
function powerShellCommand(command, requiredFile, exit) {
    initPlugin();
    if (getFileExists(currentDirName + requiredFile)) {
			if(exit) {
				ExecuteFile(Script.Path + "ps_execute.cmd", "\""+currentDirName+"\" \""+command+"\"");
			} else {
				ExecuteFile(Script.Path + "ps_command.cmd", "\""+currentDirName+"\" \""+command+"\"");
			}
    } else {
        alert("Sorry, cant't find <"+requiredFile+"> in active Folder. ")
    }
}

/**
 * Open Powershell with right filetype
 */
function powerShellCommand4Script2Ext(fileExt, command) {
    initPlugin();
    if (getFileExt(fileExt)) {
        ExecuteFile(Script.Path + "ps_execute.cmd", "\""+currentDirName+"\" \""+command+"\" \""+fileName+"\"");
    } else {
        alert("Filetype not supported!");
    }
}

/**
 * Execute Powershell-Command
 */
function powerShellExecute(command, exit) {
    initPlugin();
		if(exit) {
			ExecuteFile(Script.Path + "ps_execute.cmd", "\""+currentDirName+"\" \""+command+"\"");
		} else {
			ExecuteFile(Script.Path + "ps_command.cmd", "\""+currentDirName+"\" \""+command+"\"");
		}
}

/**
 * Execute Powershell-Command with required File in workdir
 */
function powerShellExecuteCommand(command, requiredFile, exit) {
    initPlugin();
		if (getFileExists(currentDirName + requiredFile)) {
			var ask = prompt("Enter your Command!", "watch");
			if(exit) {
				ExecuteFile(Script.Path + "ps_execute.cmd", "\""+currentDirName+"\" \""+command+"\" \""+ask+"\"");
			} else {
				ExecuteFile(Script.Path + "ps_command.cmd", "\""+currentDirName+"\" \""+command+"\" \""+ask+"\"");
			}
		} else {
        alert("Sorry, cant't find <"+requiredFile+"> in active Folder. ")
    }
}

/**
 * Functions
 */
function openPSnpminstall() { powerShellCommand("npm install", "package.json", true); }
function openPSbowerinstall() { powerShellCommand("bower install", "bower.json", true); }
function openPScomposerinstall() { powerShellCommand("composer install", "composer.json", true); }
function openPSgrunt() { powerShellCommand("grunt", "gruntfile.js", true); }
function openPSgruntwatch() { powerShellExecuteCommand("grunt", "gruntfile.js", true); }
function openPSgulp() { powerShellCommand("gulp", "gulpfile.js", true); }
function openPSgulpwatch() { powerShellExecuteCommand("gulp", "gulpfile.js", true); }
function openPS4tscfile() { powerShellCommand4Script2Ext(".ts", "tsc"); }
function openPS4Coffeefile() { powerShellCommand4Script2Ext(".coffee", "coffee -c"); }
function openPS4beautyJSFile() { powerShellCommand4Script2Ext(".js,.json", "js-beautify --config \""+Script.Path+"beautify.json\" --type \"js\" -r" ); }
function openPS4beautyHTMLFile() { powerShellCommand4Script2Ext(".html,.htm,.xhtml,.tpl", "js-beautify --config \""+Script.Path+"beautify.json\" --type \"html\" -r" ); }
function openPS4beautyCSSFile() { powerShellCommand4Script2Ext(".css", "js-beautify --config \""+Script.Path+"beautify.json\" --type \"css\" -r" ); }
function openPS4minifierJSFile() { powerShellCommand4Script2Ext(".js", "minify --clean " ); }
function openPS4minifierCSSFile() { powerShellCommand4Script2Ext(".css", "minify --clean "); }
function openPSyeomanInstall() { powerShellExecute("yo robonkey", true); }
function openPSyeomanInstallExpress() { powerShellExecute("yo robonkey:express", true); }
function openPSyeomanInstallWordpress() { powerShellExecute("yo robonkey:wordpress", true); }
function openPSyeomanInstallDrupal() { powerShellExecute("yo robonkey:drupal", true); }
function openPSyeomanInstallCodeigniter() { powerShellExecute("yo robonkey:codeigniter", true); }
function openPSyeomanInstallLaravel() { powerShellExecute("yo robonkey:laravel", true); }
function openPSyeomanInstallAngularJS1() { powerShellExecute("yo gulp-angular", true); }
function openPSyeomanDoctor() { powerShellExecute("yo doctor", false); }

/**
 * Split Sting
 */
function split(str, splitChar) {
    var res[0],count,val,p;
    count = 0;
    val = str + splitChar;
	p = Pos(splitChar,val);
	while (p > 0) {
		arg = Trim(Copy(val, 1, p - 1));
		DeleteStr(val, 1, p)
	 	SetLength(res,count + 1);
		res[count] = arg;
		count++;
		p = Pos(splitChar, val);
	}
	return res;
}

/**
 * Show info when plugin is installed
 */
function isInstalled() {
    initPlugin();
    var initalQuestion = confirm("You have Node.js installed on your local computer?\n\nThis WeBuilder-Plugin can't install it for you!");
    if (initalQuestion) {
        var ask = confirm("You like to install the required Node.js modules?\n\nModules to install:\n\nbower\ncomposer\ngrunt\ngulp\ntypescript\ncoffee-script\njs-beautify\nminifier\nyeoman\ngenerator-robonkey\ngenerator-gulp-angular\n");
        if (ask) {
            ExecuteFile(Script.Path + "ps_execute.cmd", "\""+currentDirName+"\" \"powershell -ExecutionPolicy Bypass -File "+Script.Path+"install-modules.ps1\"");
            alert("PowerShell-Plugin v1.26 beta \nby Frank Butenhoff (fkbff) \ninstalled sucessfully!");
        } else {
            alert("You can't use this Plugin whitout the required Node.js modules.\n\nPlease install:\n\nbower\ncomposer\ngrunt\ngulp\ntypescript\ncoffee-script\njs-beautify\nminifier\nyeoman\ngenerator-robonkey\ngenerator-gulp-angular\n")
        }
    } else {
        alert("You can't use this Plugin whitout Node.js\n\nPlease install Node.js and try it again!");
        ExecuteFile(Script.Path + "ps_execute.cmd", "\""+currentDirName+"\" \"Start-Process -FilePath https://nodejs.org/en/download/\"");
    }    
}

/**
 * Helper-Function
 */
function setMenue(pic, command, scriptFunc, bitmap) {
    initPlugin();
    LoadFileToBitmap(Script.Path + pic, bitmap);
    actionPic = Script.RegisterDocumentAction("PowerShell", command, "", scriptFunc);
    Actions.SetIcon(actionPic, bitmap);
}

Script.ConnectSignal("installed", "isInstalled");

/**
 * Set Menuitems
 */
setMenue("powershell.png", "PowerShell", "powerShell", bitmap);
setMenue("npm.png", "npm install", "openPSnpminstall", bitmap);
setMenue("bower.png", "bower install", "openPSbowerinstall", bitmap);
setMenue("composer.png", "composer install", "openPScomposerinstall", bitmap);
setMenue("grunt.png", "grunt", "openPSgrunt", bitmap);
setMenue("grunt.png", "grunt <commad>", "openPSgruntwatch", bitmap);
setMenue("gulp.png", "gulp", "openPSgulp", bitmap);
setMenue("gulp.png", "gulp <command>", "openPSgulpwatch", bitmap);
setMenue("typescript.png", "compile tsc <file>", "openPS4tscfile", bitmap);
setMenue("coffee.png", "compile coffee-script <file>", "openPS4Coffeefile", bitmap);
setMenue("beauty-js.png", "beautify js <file>", "openPS4beautyJSFile", bitmap);
setMenue("beauty-css.png", "beautify css <file>", "openPS4beautyCSSFile", bitmap); 
setMenue("beauty-html.png", "beautify html <file>", "openPS4beautyHTMLFile", bitmap);
setMenue("minify-js.png", "minify js <file>", "openPS4minifierJSFile", bitmap);
setMenue("minify-css.png", "minify css <file>", "openPS4minifierCSSFile", bitmap);
setMenue("yeoman.png", "yeoman Install JS Projekt<dir>", "openPSyeomanInstall", bitmap);
setMenue("yeoman.png", "yeoman Install Express <dir>", "openPSyeomanInstallExpress", bitmap);
setMenue("yeoman.png", "yeoman Install Wordpress <dir>", "openPSyeomanInstallWordpress", bitmap);
setMenue("yeoman.png", "yeoman Install Drupal <dir>", "openPSyeomanInstallDrupal", bitmap);
setMenue("yeoman.png", "yeoman Install Codeigniter <dir>", "openPSyeomanInstallCodeigniter", bitmap);
setMenue("yeoman.png", "yeoman Install Laravel <dir>", "openPSyeomanInstallLaravel", bitmap);
setMenue("yeoman.png", "yeoman Install AngularJS <dir>", "openPSyeomanInstallAngularJS1", bitmap);
setMenue("yeoman.png", "yeoman doctor <dir>", "openPSyeomanDoctor", bitmap);

delete bitmap;